/**********************************************************************
*  2007 Microchip Technology Inc.
*
* FileName:        main.c
* Dependencies:    Header (.h) files if applicable, see below
*
* SOFTWARE LICENSE AGREEMENT:
* Microchip Technology Incorporated ("Microchip") retains all ownership and 
* intellectual property rights in the code accompanying this message and in all 
* derivatives hereto.  You may use this code, and any derivatives created by 
* any person or entity by or on your behalf, exclusively with Microchip,s 
* proprietary products.  Your acceptance and/or use of this code constitutes 
* agreement to the terms and conditions of this notice.
*
* CODE ACCOMPANYING THIS MESSAGE IS SUPPLIED BY MICROCHIP "AS IS".  NO 
* WARRANTIES, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED 
* TO, IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS FOR A 
* PARTICULAR PURPOSE APPLY TO THIS CODE, ITS INTERACTION WITH MICROCHIP,S 
* PRODUCTS, COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN ANY APPLICATION. 
*
* YOU ACKNOWLEDGE AND AGREE THAT, IN NO EVENT, SHALL MICROCHIP BE LIABLE, WHETHER 
* IN CONTRACT, WARRANTY, TORT (INCLUDING NEGLIGENCE OR BREACH OF STATUTORY DUTY), 
* STRICT LIABILITY, INDEMNITY, CONTRIBUTION, OR OTHERWISE, FOR ANY INDIRECT, SPECIAL, 
* PUNITIVE, EXEMPLARY, INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, FOR COST OR EXPENSE OF 
* ANY KIND WHATSOEVER RELATED TO THE CODE, HOWSOEVER CAUSED, EVEN IF MICROCHIP HAS BEEN 
* ADVISED OF THE POSSIBILITY OR THE DAMAGES ARE FORESEEABLE.  TO THE FULLEST EXTENT 
* ALLOWABLE BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN ANY WAY RELATED TO 
* THIS CODE, SHALL NOT EXCEED THE PRICE YOU PAID DIRECTLY TO MICROCHIP SPECIFICALLY TO 
* HAVE THIS CODE DEVELOPED.
*
* You agree that you are solely responsible for testing the code and 
* determining its suitability.  Microchip has no obligation to modify, test, 
* certify, or support the code.
*
* REVISION HISTORY:
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* Author          	Date      Comments on this revision
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* Jatinder Gharoo 	10/30/08  First release of source file
* 
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*
* HARDWARE USED: 
* Explorer-16 development board revision R5
* ECAN/LIN PICTail Plus Daughter board revision R2
* dsPIC33FJ256GP710 PIM 
* testing using PIC24H256GP610 PIM 
* 
* SOFTWARE USED: 
* MPLAB IDE v8.14
* MPLAB C Compiler for PIC24 MCUs and dsPIC DSCs v 3.11(b) 
* 
* CAN CONFIGURATION USED:
* Bit Time = (Sync Segment + Propagation Delay + Phase Segment 1 + Phase Segment 2)=20TQ
* Phase Segment 1 = 8TQ
* Phase Segment 2 = 6Tq
* Propagation Delay = 5Tq
* Sync Segment = 1TQ
* BIT RATE OF 1Mbps
*	
* ADDITIONAL NOTES:
* Oscillator 8Mhz mounted on the Explorer 16 board 
* CAN Bus speed of 1Mhz u
* USB CAN sniffer used ValueCAN from Intrepid Control Systems for testing
* MPLAB REAL ICE used for Debugging  
* 
*************************************************************************************************/


/* Include the appropriate header (.h) file, depending on device used */

#if defined(__dsPIC33F__)
#include <p33Fxxxx.h>
#elif defined(__PIC24H__)
#include <p24hxxxx.h>
#endif

#include "ecan.h"
#include "delay.h"

/*****************************************************************************************************
Configuration bits: 
_FOSCSEL: Start up with the FRC oscillator 
_FOSC: Use Primary oscillator XT w/ PLL, Two-speed Oscillator Startup enabled 
_FWDT: Watch dog timer is off
_FPOR: Power-on Reset Value disabled, Alternate I2C pins mapped to SDA1/SCL1
_FICD: JTAG is disabled,  communicate on PGC3/EMUC3 and PGD3/EMUD3 for explorer 16 development board  	
*****************************************************************************************************/
_FOSCSEL(FNOSC_FRC) 
_FOSC(FCKSM_CSECMD & POSCMD_XT)
_FWDT(FWDTEN_OFF)
_FPOR(FPWRT_PWR1)
_FICD(JTAGEN_OFF & ICS_PGD1)
        
/*****************************************************************************************************
Function Prototypes  
*****************************************************************************************************/
void oscConfig(void);

/*****************************************************************************************************
Globals   
*****************************************************************************************************/
mID canTxMessage;
mID canRxMessage;

/* Define ECAN Message Buffers */
ECAN1MSGBUF ecan1msgBuf __attribute__((space(dma),aligned(ECAN1_MSG_BUF_LENGTH*16)));


/*****************************************************************************************************
main() function 
*****************************************************************************************************/
int main ( void )
{	
	/* Configure Oscillator Clock Source */
	oscConfig();
	
	/* initialise other modules */				
	initECAN();
	initDMAECAN();
	
	/* Enable ECAN1 Interrupt */     	
	IEC2bits.C1IE=1;	
	/* enable Transmit interrupt */
	C1INTEbits.TBIE=1;
	/* Enable Receive interrupt */
	C1INTEbits.RBIE=1;

#ifdef THIS_IS_THE_TRANSMITTER
	/* configure and send a message */
	canTxMessage.message_type=CAN_MSG_DATA;
	//canTxMessage.message_type=CAN_MSG_RTR;
//	canTxMessage.frame_type=CAN_FRAME_EXT;
	canTxMessage.frame_type=CAN_FRAME_STD;
	canTxMessage.buffer=0;
	canTxMessage.id=0x123;
	canTxMessage.data[0]=0x55;
	canTxMessage.data[1]=0x55;
	canTxMessage.data[2]=0x55;
	canTxMessage.data[3]=0x55;
	canTxMessage.data[4]=0x55;
	canTxMessage.data[5]=0x55;
	canTxMessage.data[6]=0x55;
	canTxMessage.data[7]=0x55;
	canTxMessage.data_length=8;
	
	/* Delay for a second */
	Delay(Delay_1S_Cnt);
		
	/* send a CAN message */
	sendECAN(&canTxMessage);
#endif

	while(1)
	{
#if defined THIS_IS_THE_RECEIVER
		/* check to see when a message is received and move the message 
		into RAM and parse the message */ 
		if(canRxMessage.buffer_status==CAN_BUF_FULL)
		{
			rxECAN(&canRxMessage);
			/* reset the flag when done */
			canRxMessage.buffer_status=CAN_BUF_EMPTY;
		}
#elif defined THIS_IS_THE_TRANSMITTER

		// delay for one second
		Delay(Delay_1S_Cnt);
		// send another message
		sendECAN(&canTxMessage);
#endif
	}
}

/*****************************************************************************************************
oscConfig() function 
*****************************************************************************************************/
void oscConfig(void){

	/*  Configure Oscillator to operate the device at 40MIPS
 	Fosc= Fin*M/(N1*N2), Fcy=Fosc/2
 	Fosc= 8M*40/(2*2)=80Mhz for 8M input clock */

	PLLFBD=38;					/* M=40 */
	CLKDIVbits.PLLPOST=0;		/* N1=2 */
	CLKDIVbits.PLLPRE=0;		/* N2=2 */
	OSCTUN=0;					/* Tune FRC oscillator, if FRC is used */
	
	/* Disable Watch Dog Timer */
	RCONbits.SWDTEN=0;
	
	/* Clock switch to incorporate PLL*/
	__builtin_write_OSCCONH(0x03);		// Initiate Clock Switch to Primary
										// Oscillator with PLL (NOSC=0b011)
	__builtin_write_OSCCONL(0x01);		// Start clock switching
	while (OSCCONbits.COSC != 0b011);	// Wait for Clock switch to occur

	/* Wait for PLL to lock */
	while(OSCCONbits.LOCK!=1) {};
}

/****** START OF INTERRUPT SERVICE ROUTINES *********/

/* Replace the interrupt function names with the    */
/* appropriate names depending on interrupt source. */

/* The names of various interrupt functions for     */
/* each device are defined in the linker script.    */


/* Interrupt Service Routine 1                      */
/* No fast context save, and no variables stacked   */
void __attribute__((interrupt,no_auto_psv))_C1Interrupt(void)  
{
	/* check to see if the interrupt is caused by receive */     	 
    if(C1INTFbits.RBIF)
    {
	    /* check to see if buffer 1 is full */
	    if(C1RXFUL1bits.RXFUL1)
	    {			
			/* set the buffer full flag and the buffer received flag */
			canRxMessage.buffer_status=CAN_BUF_FULL;
			canRxMessage.buffer=1;	
		}		
		/* check to see if buffer 2 is full */
		else if(C1RXFUL1bits.RXFUL2)
		{
			/* set the buffer full flag and the buffer received flag */
			canRxMessage.buffer_status=CAN_BUF_FULL;
			canRxMessage.buffer=2;					
		}
		/* check to see if buffer 3 is full */
		else if(C1RXFUL1bits.RXFUL3)
		{
			/* set the buffer full flag and the buffer received flag */
			canRxMessage.buffer_status=CAN_BUF_FULL;
			canRxMessage.buffer=3;					
		}
		else;
		/* clear flag */
		C1INTFbits.RBIF = 0;
	}
	else if(C1INTFbits.TBIF)
    {
	    /* clear flag */
		C1INTFbits.TBIF = 0;	    
	}
	else;
	
	/* clear interrupt flag */
	IFS2bits.C1IF=0;
}

/********* END OF INTERRUPT SERVICE ROUTINES ********/
